@@ -739,3 +739,20 @@ orc_arm_emit_data (OrcCompiler *compiler, orc_uint32 data)
   }
 }
 
+void
+orc_arm_loadw (OrcCompiler *compiler, int dest, int src1, int offset)
+{
+  orc_uint32 code;
+
+  code = 0xe1d000b0;
+  code |= (src1&0xf) << 16;
+  code |= (dest&0xf) << 12;
+  code |= (offset&0xf0) << 4;
+  code |= offset&0x0f;
+
+  ORC_ASM_CODE(compiler,"  ldrh %s, [%s, #%d]\n",
+      orc_arm_reg_name (dest),
+      orc_arm_reg_name (src1), offset);
+  orc_arm_emit (compiler, code);
+}
+
