@@ -318,7 +318,7 @@ mmx_save_accumulators (OrcCompiler *compiler)
         }
 
         if (compiler->vars[i].size == 2) {
-          orc_x86_emit_mov_mmx_reg (compiler, src, compiler->gp_tmpreg);
+          orc_mmx_emit_movd_store_register (compiler, src, compiler->gp_tmpreg);
           orc_x86_emit_and_imm_reg (compiler, 4, 0xffff, compiler->gp_tmpreg);
           orc_x86_emit_mov_reg_memoffset (compiler, 4, compiler->gp_tmpreg,
               (int)ORC_STRUCT_OFFSET(OrcExecutor, accumulators[i-ORC_VAR_A1]),
@@ -429,7 +429,7 @@ orc_mmx_load_constant (OrcCompiler *compiler, int reg, int size, orc_uint64 valu
   }
 
   orc_x86_emit_mov_imm_reg (compiler, 4, value, compiler->gp_tmpreg);
-  orc_x86_emit_mov_reg_mmx (compiler, compiler->gp_tmpreg, reg);
+  orc_mmx_emit_movd_load_register (compiler, compiler->gp_tmpreg, reg);
 #ifndef MMX
   orc_mmx_emit_pshufd (compiler, ORC_MMX_SHUF(0,0,0,0), reg, reg);
 #else
@@ -482,7 +482,7 @@ mmx_load_constants_outer (OrcCompiler *compiler)
       case ORC_VAR_TYPE_TEMP:
         break;
       default:
-        ORC_COMPILER_ERROR(compiler,"bad vartype");
+        orc_compiler_error(compiler,"bad vartype");
         break;
     }
   }
@@ -555,7 +555,7 @@ mmx_load_constants_inner (OrcCompiler *compiler)
       case ORC_VAR_TYPE_TEMP:
         break;
       default:
-        ORC_COMPILER_ERROR(compiler,"bad vartype");
+        orc_compiler_error(compiler,"bad vartype");
         break;
     }
   }
@@ -583,7 +583,7 @@ mmx_add_strides (OrcCompiler *compiler)
             (int)ORC_STRUCT_OFFSET(OrcExecutor, arrays[i]), compiler->exec_reg);
 
         if (compiler->vars[i].ptr_register == 0) {
-          ORC_COMPILER_ERROR(compiler, "unimplemented: stride on mem pointer");
+          orc_compiler_error (compiler, "unimplemented: stride on pointer stored in memory");
         }
         break;
       case ORC_VAR_TYPE_ACCUMULATOR:
@@ -591,7 +591,7 @@ mmx_add_strides (OrcCompiler *compiler)
       case ORC_VAR_TYPE_TEMP:
         break;
       default:
-        ORC_COMPILER_ERROR(compiler,"bad vartype");
+        orc_compiler_error(compiler,"bad vartype");
         break;
     }
   }
@@ -618,7 +618,7 @@ get_align_var (OrcCompiler *compiler)
     return i;
   }
 
-  ORC_COMPILER_ERROR(compiler, "could not find alignment variable");
+  orc_compiler_error(compiler, "could not find alignment variable");
 
   return -1;
 }
@@ -713,7 +713,7 @@ static void
 orc_emit_split_2_regions (OrcCompiler *compiler)
 {
   int align_var;
-  int align_shift;
+  int align_shift ORC_GNUC_UNUSED;
   int var_size_shift;
 
   align_var = get_align_var (compiler);
@@ -768,6 +768,7 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
   int set_mxcsr = FALSE;
 #endif
   int align_var;
+  int is_aligned;
 
   if (0 && orc_x86_assemble_copy_check (compiler)) {
     /* The rep movs implementation isn't faster most of the time */
@@ -776,8 +777,7 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
   }
 
   align_var = get_align_var (compiler);
-
-  compiler->vars[align_var].is_aligned = FALSE;
+  is_aligned = compiler->vars[align_var].is_aligned;
 
   {
     orc_mmx_emit_loop (compiler, 0, 0);
@@ -787,7 +787,9 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
     compiler->asm_code = NULL;
     compiler->asm_code_len = 0;
     memset (compiler->labels, 0, sizeof (compiler->labels));
+    memset (compiler->labels_int, 0, sizeof (compiler->labels_int));
     compiler->n_fixups = 0;
+    compiler->n_output_insns = 0;
   }
 
   if (compiler->error) return;
@@ -828,11 +830,11 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
       compiler->program->constant_n <= ORC_MMX_ALIGNED_DEST_CUTOFF) {
     /* don't need to load n */
   } else if (compiler->loop_shift > 0) {
-    if (!compiler->has_iterator_opcode) {
+    if (compiler->has_iterator_opcode || is_aligned) {
+      orc_emit_split_2_regions (compiler);
+    } else {
       /* split n into three regions, with center region being aligned */
       orc_emit_split_3_regions (compiler);
-    } else {
-      orc_emit_split_2_regions (compiler);
     }
   } else {
     /* loop shift is 0, no need to split */
@@ -877,7 +879,7 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
     int emit_region1 = TRUE;
     int emit_region3 = TRUE;
 
-    if (compiler->has_iterator_opcode) {
+    if (compiler->has_iterator_opcode || is_aligned) {
       emit_region1 = FALSE;
     }
     if (compiler->loop_shift == 0) {
@@ -920,7 +922,7 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
     }
 
     ORC_ASM_CODE(compiler, "# LOOP SHIFT %d\n", compiler->loop_shift);
-    orc_x86_emit_align (compiler);
+    orc_x86_emit_align (compiler, 4);
     orc_x86_emit_label (compiler, LABEL_INNER_LOOP_START);
     ui_max = 1<<compiler->unroll_shift;
     for(ui=0;ui<ui_max;ui++) {
@@ -982,6 +984,9 @@ orc_compiler_mmx_assemble (OrcCompiler *compiler)
 #endif
   orc_x86_emit_epilogue (compiler);
 
+  orc_x86_calculate_offsets (compiler);
+  orc_x86_output_insns (compiler);
+
   orc_x86_do_fixups (compiler);
 }
 
@@ -1019,13 +1024,20 @@ orc_mmx_emit_loop (OrcCompiler *compiler, int offset, int update)
       if (!(insn->opcode->flags & (ORC_STATIC_OPCODE_ACCUMULATOR|ORC_STATIC_OPCODE_LOAD|ORC_STATIC_OPCODE_STORE)) &&
           compiler->vars[insn->dest_args[0]].alloc !=
           compiler->vars[insn->src_args[0]].alloc) {
-        orc_x86_emit_mov_mmx_reg_reg (compiler,
+#ifdef MMX
+        orc_mmx_emit_movq (compiler,
             compiler->vars[insn->src_args[0]].alloc,
             compiler->vars[insn->dest_args[0]].alloc);
+#else
+        orc_mmx_emit_movdqu (compiler,
+            compiler->vars[insn->src_args[0]].alloc,
+            compiler->vars[insn->dest_args[0]].alloc);
+#endif
       }
       rule->emit (compiler, rule->emit_user, insn);
     } else {
-      ORC_COMPILER_ERROR(compiler,"No rule for: %s", opcode->name);
+      orc_compiler_error (compiler, "no code generation rule for %s",
+          opcode->name);
     }
   }
 
@@ -1090,7 +1102,8 @@ orc_mmx_emit_invariants (OrcCompiler *compiler)
     if (rule && rule->emit) {
       rule->emit (compiler, rule->emit_user, insn);
     } else {
-      ORC_COMPILER_ERROR(compiler,"No rule for: %s", opcode->name);
+      orc_compiler_error (compiler, "no code generation rule for %s",
+          opcode->name);
     }
   }
 }
